'2009 IDesign Inc. 
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Collections

Namespace ServiceModelEx
    Public NotInheritable Class Collection
        ''' <summary>
        ''' Bypasses elements in a collection as long as a specified condition is true
        '  and then returns the remaining elements.
        ''' </summary>
        Private Sub New()
        End Sub
        Public Shared Function SkipWhile(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            Dim items As New List(Of T)
            For Each item As T In collection
                If match(item) = False Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function

        ''' <summary>
        ''' Returns a specified number of contiguous elements from the start of a collection.
        ''' </summary>
        Public Shared Function Take(Of T)(ByVal collection As IEnumerable(Of T), ByVal count As Integer) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If count < 0 Then
                Throw New IndexOutOfRangeException("count")
            End If
            Dim i As Integer = 0
            Dim items As New List(Of T)
            For Each item As T In collection
                i += 1
                If i > count Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function

        ''' <summary>
        ''' Returns elements from a collection as long as a specified condition is true.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns>An System.Collections.Generic.IEnumerable<T> that contains the elements from
        '     the input collection that occur before the element at which the test no longer
        '     passes.</returns>
        Public Shared Function TakeWhile(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            Dim items As New List(Of T)
            For Each item As T In collection
                If match(item) Then
                    Return items
                End If
                items.Add(item)
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Determines whether two collections are equal by comparing the elements by using   
        ''' </summary>
        Public Shared Function SequenceEqual(Of T As IEquatable(Of T))(ByVal first As IEnumerable(Of T), ByVal second As IEnumerable(Of T)) As Boolean
            If first Is Nothing AndAlso second Is Nothing Then
                Return True
            End If
            If first Is Nothing Then
                Throw New ArgumentNullException("first")
            End If
            If second Is Nothing Then
                Throw New ArgumentNullException("second")
            End If
            If Count(first) <> Count(second) Then
                Return False
            End If
            Using enumerator1 As IEnumerator(Of T) = first.GetEnumerator()
                Using enumerator2 As IEnumerator(Of T) = second.GetEnumerator()

                    Dim count As Integer = Collection.Count(first)

                    Dim index As Integer = 1
                    Do While enumerator1.Current.Equals(enumerator2.Current)
                        index += 1
                        Continue Do
                    Loop
                    Return index = count
                End Using
            End Using
        End Function
        ''' <summary>
        ''' Bypasses a specified number of elements in a collection and then returns the
        ''' </summary>
        Public Shared Function Skip(Of T)(ByVal collection As IEnumerable(Of T), ByVal count As Integer) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If count < 0 Then
                Throw New IndexOutOfRangeException("count")
            End If
            Dim i As Integer = 0
            Dim items As New List(Of T)
            For Each item As T In collection
                i += 1
                If i > count Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function


        ''' <summary>
        ''' Generates a collection that contains one repeated value.
        ''' </summary>
        Public Shared Function Repeat(Of T)(ByVal element As T, ByVal count As Integer) As IEnumerable(Of T)
            If count <= 0 Then
                Throw New ArgumentOutOfRangeException("count")
            End If
            Dim items As New List(Of T)
            For i As Integer = 0 To count - 1
                items.Add(element)
            Next i
            Return items
        End Function
        ''' <summary>
        ''' Returns the element at a specified index in a collection.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection">An System.Collections.Generic.IEnumerable<T> to return an element from</param>
        ''' <param name="index">The zero-based index of the element to retrieve.</param>
        ''' <returns></returns>
        Public Shared Function ElementAt(Of T)(ByVal collection As IEnumerable(Of T), ByVal index As Integer) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If index < 0 OrElse index > Count(collection) - 1 Then
                Throw New ArgumentOutOfRangeException("index")
            End If
            Return ToArray(collection)(index)
        End Function

        ''' <summary>
        ''' Returns the element at a specified index in a collection.
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection">An System.Collections.Generic.IEnumerable<T> to return an element from</param>
        ''' <param name="index">The zero-based index of the element to retrieve.</param>
        ''' <returns></returns>
        Public Shared Function ElementAtOrDefault(Of T)(ByVal collection As IEnumerable(Of T), ByVal index As Integer) As T
            If index < 0 OrElse index > Count(collection) - 1 OrElse collection Is Nothing Then
                Return Nothing
            End If
            Return ElementAt(collection, index)
        End Function
        ''' <summary>
        ''' Concatenates two collections.
        ''' </summary>
        Public Shared Function Concat(Of T)(ByVal first As IEnumerable(Of T), ByVal second As IEnumerable(Of T)) As IEnumerable(Of T)
            If first Is Nothing Then
                Throw New ArgumentNullException("first")
            End If
            If second Is Nothing Then
                Throw New ArgumentNullException("second")
            End If
            Dim items As New List(Of T)
            For Each item As T In first
                items.Add(item)
            Next item
            For Each item As T In second
                items.Add(item)
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Returns true if collection contains the item
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="item"></param>
        ''' <returns></returns>
        Public Shared Function Contains(Of T As IEquatable(Of T))(ByVal collection As IEnumerable(Of T), ByVal item As T) As Boolean
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            For Each i As T In collection
                If i.Equals(item) Then
                    Return True
                End If
            Next i
            Return False
        End Function
        ''' <summary>
        ''' Converts all the items of type T in collection to a new collection of type U according to converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <typeparam name="U"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="converter"></param>
        ''' <returns></returns>
        Public Shared Function ConvertAll(Of T, U)(ByVal collection As IEnumerable(Of T), ByVal converter As Converter(Of T, U)) As IEnumerable(Of U)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim items As New List(Of U)
            For Each item As T In collection
                items.Add(converter(item))
            Next item
            Return items
        End Function

        ''' <summary>
        ''' Returns true if collection contains an item that satisfies the predicate
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function Any(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As Boolean
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            For Each item As T In collection
                If match(item) Then
                    Return True
                End If
            Next item
            Return False
        End Function
        ''' <summary>
        ''' Determines whether a collection contains any elements
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function Any(Of T)(ByVal collection As IEnumerable(Of T)) As Boolean
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Using iterator As IEnumerator(Of T) = collection.GetEnumerator()
                Return iterator.MoveNext()
            End Using
        End Function
        ''' <summary>
        ''' Finds the only item in the collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function [Single](Of T)(ByVal collection As IEnumerable(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If Count(collection) <> 1 Then
                Throw New InvalidOperationException("collection")
            End If
            Return ToList(collection)(0)
        End Function


        ''' <summary>
        ''' Finds the only item in the collection that satisfies the predicate
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function [Single](Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            If Count(collection) = 0 Then
                Throw New InvalidOperationException("collection")
            End If
            Dim list As IList(Of T) = ToList(FindAll(collection, match))
            If list.Count <> 1 Then
                Throw New InvalidOperationException("Multiple matches found")
            End If
            Return list(0)
        End Function



        ''' <summary>
        ''' Finds the only item in the collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function SingleOrDefault(Of T)(ByVal collection As IEnumerable(Of T)) As T
            If collection Is Nothing OrElse Count(collection) <> 1 Then
                Return Nothing
            End If
            Return [Single](collection)
        End Function


        ''' <summary>
        ''' Finds the only item in the collection that satisfies the predicate
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function SingleOrDefault(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As T
            If collection Is Nothing OrElse match Is Nothing Then
                Return Nothing
            End If
            If Count(collection) = 0 Then
                Return Nothing
            End If
            Dim list As IList(Of T) = ToList(FindAll(collection, match))
            If list.Count <> 1 Then
                Return Nothing
            End If
            Return list(0)
        End Function
        ''' <summary>
        ''' Finds the first item in the collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function First(Of T)(ByVal collection As IEnumerable(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If Count(collection) = 0 Then
                Throw New InvalidOperationException("collection")
            End If
            Return ToList(collection)(0)
        End Function

        ''' <summary>
        ''' Finds the first item in the collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function FirstOrDefault(Of T)(ByVal collection As IEnumerable(Of T)) As T
            If Count(collection) = 0 OrElse collection Is Nothing Then
                Return Nothing
            End If
            Return ToList(collection)(0)
        End Function

        ''' <summary>
        ''' Finds the first occurrence of item in collection that satisfy the predicate
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function First(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            For Each item As T In collection
                If match(item) Then
                    Return item
                End If
            Next item
            Return Nothing
        End Function
        ''' <summary>
        ''' Finds all the items in collection that satisfy the predicate match. Same as LINQ's Where
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function FindAll(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            Dim items As New List(Of T)
            For Each item As T In collection
                If match(item) Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Finds all the items in iterator1 that are not in collection2
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator1"></param>
        ''' <param name="collection2"></param>
        ''' <returns></returns>
        Public Shared Function Complement(Of T As IEquatable(Of T))(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim items As New List(Of T)
            For Each item As T In collection1
                If Contains(collection2, item) = False Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function

        ''' <summary>
        ''' Finds all the items that are not in the intersection of collection1 and collection2
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection1"></param>
        ''' <param name="collection2"></param>
        ''' <returns></returns>
        Public Shared Function Except(Of T As IEquatable(Of T))(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim complement1 As IEnumerable(Of T) = Complement(collection1, collection2)
            Dim complement2 As IEnumerable(Of T) = Complement(collection2, collection1)
            Return Union(complement1, complement2)
        End Function
        ''' <summary>
        ''' Returns a collection of all the distinct items in collection (no duplicates)
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function Distinct(Of T)(ByVal collection As IEnumerable(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As IList(Of T) = New List(Of T)()
            For Each item As T In collection
                If list.Contains(item) = False Then
                    list.Add(item)
                End If
            Next item
            Dim items As New List(Of T)
            For Each item As T In list
                items.Add(item)
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Find the index of the first occurrence of item in collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="value"></param>
        ''' <returns></returns>
        Public Shared Function FindIndex(Of T As IEquatable(Of T))(ByVal collection As IEnumerable(Of T), ByVal value As T) As Integer
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim index As Integer = 0

            For Each item As T In collection
                If item.Equals(value) = False Then
                    index += 1
                Else
                    Return index
                End If
            Next item
            Return -1
        End Function
        ''' <summary>
        ''' Finds the intersection of collection1 and collection2
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection1"></param>
        ''' <param name="collection2"></param>
        ''' <returns></returns>
        Public Shared Function Intersect(Of T As IEquatable(Of T))(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim items As New List(Of T)
            For Each item1 In collection1
                For Each item2 In collection2
                    If item1.Equals(item2) Then
                        items.Add(item2)
                    End If
                Next
            Next
            Return items
        End Function

        ''' <summary>
        ''' Find the index of the last occurrence of item in collection that satisfy the predicate match
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function LastOrDefault(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As T
            If collection Is Nothing Then
                Return Nothing
            End If
            If Count(collection) = 0 Then
                Return Nothing
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            Return Last(collection, match)
        End Function

        ''' <summary>
        ''' Find the index of the last occurrence of item in collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function LastOrDefault(Of T)(ByVal collection As IEnumerable(Of T)) As T
            If collection Is Nothing Then
                Return Nothing
            End If
            If Count(collection) = 0 Then
                Return Nothing
            End If
            Return Last(collection)
        End Function
        ''' <summary>
        ''' Find the index of the last occurrence of item in collection 
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function Last(Of T)(ByVal collection As IEnumerable(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Return ToArray(collection)(Count(collection) - 1)
        End Function

        ''' <summary>
        ''' Find the index of the last occurrence of item in collection that satisfy the predicate match
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function Last(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As T
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            Dim lastItem As T = Nothing

            For Each item As T In collection
                If match(item) Then
                    lastItem = item
                End If
            Next item
            Return lastItem
        End Function
        ''' <summary>
        ''' Find the index of the last occurrence of item in collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="value"></param>
        ''' <returns></returns>
        Public Shared Function FindLastIndex(Of T As IEquatable(Of T))(ByVal collection As IEnumerable(Of T), ByVal value As T) As Integer
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim last As Integer = -1

            For Each item As T In collection
                If item.Equals(value) Then
                    last += 1
                End If
            Next item
            If last >= 0 Then
                Return last
            Else
                Return -1
            End If
        End Function
        ''' <summary>
        ''' Finds the union of collection1 and collection2
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection1"></param>
        ''' <param name="collection2"></param>
        ''' <returns></returns>
        Public Shared Function Union(Of T As IEquatable(Of T))(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim unionList As LinkedList(Of T) = New LinkedList(Of T)(Distinct(collection1))

            Dim closure As New UnionClosure(Of T)
            closure.unionList = unionList
            Dim action As Action(Of T) = AddressOf closure.Action
            ForEach(collection2, action)
            Dim items As New List(Of T)
            For Each item As T In closure.unionList
                items.Add(item)
            Next item
            Return items
        End Function

        Private Class UnionClosure(Of T)
            Public unionList As LinkedList(Of T)

            Public Sub Action(ByVal i As T)
                If unionList.Find(i) Is Nothing Then
                    unionList.AddLast(i)
                End If
            End Sub

        End Class
        ''' <summary>
        '''  Performs the action on every item in collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="action"></param>
        Public Shared Sub ForEach(Of T)(ByVal collection As IEnumerable(Of T), ByVal action As Action(Of T))
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If action Is Nothing Then
                Throw New ArgumentNullException("action")
            End If
            For Each item As T In collection
                action(item)
            Next item
        End Sub
        ''' <summary>
        ''' Returns the number of items in collection that satisfy a condition
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function Count(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As Integer
            Dim length As Integer = 0
            Dim closure As New CountClosure(Of T)
            closure.length = length
            closure.match = match
            Dim add As Action(Of T) = AddressOf closure.Action
            ForEach(collection, add)
            Return closure.length
        End Function

        Private Class CountClosure(Of T)
            Public length As Integer
            Public match As Predicate(Of T)

            Public Sub Action(ByVal item As T)
                If match(item) Then
                    length += 1
                End If
            End Sub

        End Class
        ''' <summary>
        ''' Returns the number of items in collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function Count(Of T)(ByVal collection As IEnumerable(Of T)) As Integer
            Dim length As Integer = 0
            Dim add As Action(Of T) = Function() length = length + 1
            ForEach(collection, add)
            Return length
        End Function
        ''' <summary>
        ''' Returns a collection with a reverse order of items
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function Reverse(Of T)(ByVal collection As IEnumerable(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As List(Of T) = New List(Of T)(collection)
            list.Reverse()
            Dim items As New List(Of T)
            For Each item As T In list
                items.Add(item)
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Sorts the collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function Sort(Of T)(ByVal collection As IEnumerable(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As List(Of T) = New List(Of T)(collection)
            list.Sort()

            Dim items As New List(Of T)
            For Each item As T In list
                items.Add(item)
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Converts collection to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function ToArray(Of T)(ByVal collection As IEnumerable(Of T)) As T()
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As List(Of T) = New List(Of T)()

            For Each item As T In collection
                list.Add(item)
            Next item
            Return list.ToArray()
        End Function

        ''' <summary>
        ''' Converts iterator to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <returns></returns>
        Public Shared Function ToArray(Of T)(ByVal iterator As IEnumerator(Of T)) As T()
            If iterator Is Nothing Then
                Throw New ArgumentNullException("iterator")
            End If
            Dim list As List(Of T) = New List(Of T)()

            Do While iterator.MoveNext()
                list.Add(iterator.Current)
            Loop
            Return list.ToArray()
        End Function
        ''' <summary>
        ''' Converts the items in collection to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <param name="count">Initial size for optimization</param>
        ''' <returns></returns>
        Public Shared Function ToArray(Of T)(ByVal collection As IEnumerable(Of T), ByVal count As Integer) As T()
            If collection Is Nothing Then
                Throw New ArgumentNullException("iterator")
            End If
            Return ToArray(collection.GetEnumerator(), count)
        End Function
        ''' <summary>
        ''' Converts the items in iterator to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <param name="count">Initial size for optimization</param>
        ''' <returns></returns>
        Public Shared Function ToArray(Of T)(ByVal iterator As IEnumerator(Of T), ByVal count As Integer) As T()
            If iterator Is Nothing Then
                Throw New ArgumentNullException("iterator")
            End If
            Dim list As List(Of T) = New List(Of T)(count)

            Do While iterator.MoveNext()
                list.Add(iterator.Current)
            Loop

            Return list.ToArray()
        End Function
        ''' <summary>
        ''' Converts the items in collection to an array of type U according to the converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <typeparam name="U"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="converter"></param>
        ''' <returns></returns>
        Shared Function ToArray(Of T, U)(ByVal collection As IEnumerable(Of T), ByVal converter As Converter(Of T, U)) As U()
            Dim count As Integer = ServiceModelEx.Collection.Count(collection)
            Return ToArray(collection, converter, count)
        End Function
        ''' <summary>
        ''' Converts the items in collection to an array  of type U according to the converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <param name="count">Initial size for optimization</param>
        ''' <returns></returns>
        Shared Function ToArray(Of T, U)(ByVal collection As IEnumerable(Of T), ByVal converter As Converter(Of T, U), ByVal count As Integer) As U()
            Dim list As List(Of U) = New List(Of U)(count)
            For Each item As T In collection
                list.Add(converter(item))
            Next item
            Return list.ToArray()
        End Function
        ''' <summary>
        ''' Returns a list out of collection
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function ToList(Of T)(ByVal collection As IEnumerable(Of T)) As IList(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As IList(Of T) = New List(Of T)()
            For Each item As T In collection
                list.Add(item)
            Next item
            Return list
        End Function

        ''' <summary>
        ''' Returns all the items in collection that satisfy the predicate match
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <param name="match"></param>
        ''' <returns></returns>
        Public Shared Function All(Of T)(ByVal collection As IEnumerable(Of T), ByVal match As Predicate(Of T)) As Boolean
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If match Is Nothing Then
                Throw New ArgumentNullException("match")
            End If
            For Each item As T In collection
                If match(item) = False Then
                    Return False
                End If
            Next item
            Return True
        End Function
        ''' <summary>
        ''' Converts all the items in the object-based collection of the type T to a new array of type U according to converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeToArray(Of T, U)(ByVal collection As IEnumerable, ByVal converter As Converter(Of T, U)) As U()
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim newCollection As IEnumerable(Of U) = UnsafeConvertAll(collection, converter)
            Return ToArray(newCollection)
        End Function

        ''' <summary>
        ''' Converts all the items in the object-based iterator of the type T to a new array of type U according to converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeToArray(Of T, U)(ByVal iterator As IEnumerator, ByVal converter As Converter(Of T, U)) As U()
            If iterator Is Nothing Then
                Throw New ArgumentNullException("iterator")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim newIterator As IEnumerator(Of U) = UnsafeConvertAll(iterator, converter)
            Return ToArray(newIterator)
        End Function




















        ''' <summary>
        ''' Converts collection to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="collection"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeToArray(Of T)(ByVal collection As IEnumerable) As T()
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim iterator As IEnumerator = collection.GetEnumerator()

            Using TryCast(iterator, IDisposable)
                Return UnsafeToArray(Of T)(iterator)
            End Using
        End Function
        ''' <summary>
        ''' Converts an iterator to an array
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <param name="iterator"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeToArray(Of T)(ByVal iterator As IEnumerator) As T()
            If iterator Is Nothing Then
                Throw New ArgumentNullException("iterator")
            End If
            Dim innerConverter As Converter(Of Object, T) = Function(item As Object) CType(item, T)
            Return UnsafeToArray(iterator, innerConverter)
        End Function
        ''' <summary>
        ''' Converts all the items of type T in the object-based collection to a new collection of type U according to converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <typeparam name="U"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="converter"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeConvertAll(Of T, U)(ByVal collection As IEnumerable, ByVal converter As Converter(Of T, U)) As IEnumerable(Of U)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim items As New List(Of U)
            For Each item As Object In collection
                items.Add(converter(CType(item, T)))
            Next item
            Return items
        End Function
        ''' <summary>
        ''' Converts all the items of type T in the object-based IEnumerator to a new collection of type U according to converter
        ''' </summary>
        ''' <typeparam name="T"></typeparam>
        ''' <typeparam name="U"></typeparam>
        ''' <param name="collection"></param>
        ''' <param name="converter"></param>
        ''' <returns></returns>
        Public Shared Function UnsafeConvertAll(Of T, U)(ByVal iterator As IEnumerator, ByVal converter As Converter(Of T, U)) As IEnumerator(Of U)
            If iterator Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim items As New List(Of U)
            Do While iterator.MoveNext()
                items.Add(converter(CType(iterator.Current, T)))
            Loop
            Return items
        End Function
    End Class
End Namespace